from PyQt4 import QtCore, QtGui

from Database import Database

import shutil
import os.path
from datetime import datetime
import sys
import os.path
from Action import Action
from Link import Link
from Sentence import Sentence
from Property import Property
from threading import Thread
from ProgressDialog import ProgressDialog


class mergingThread(Thread):
    
    def __init__(self,activeDatabaseFilename,sourceDatabaseFilename,dialogWindow,doneFunction): 
        Thread.__init__(self)
        self.activeDatabaseFilename = activeDatabaseFilename
        self.sourceDatabaseFilename = sourceDatabaseFilename
        
        
        self.dialogWindow = dialogWindow
        self.doneFunction = doneFunction
        
    def run(self):
        if not self.activeDatabaseFilename == None and len(self.activeDatabaseFilename) > 0 and os.path.isfile(self.activeDatabaseFilename):
            if not self.sourceDatabaseFilename == None and len(self.sourceDatabaseFilename) > 0 and os.path.isfile(self.sourceDatabaseFilename):
                self.dialogWindow.showMessage("Preparing databases")
                adb = Database()
                sdb = Database()
                adb.openDatabase(self.activeDatabaseFilename)
                sdb.openDatabase(self.sourceDatabaseFilename)
                
                #actionConversion = dict()
                #propertyConversion = dict()
                sentenceConversion = dict()                
                
                actionList = sdb.getAllActionsTuple() #(id,action)
                propertyList = sdb.getAllPropertiesTuple() #(id,property,show)
                sentences = sdb.getAllSentences(False, False)
                
                self.dialogWindow.showMessage("Merging Actions")
                self.dialogWindow.setRange(0,len(actionList))
                counter = 0
                for action in actionList:
                    newId = adb.addNewAction(action[1])
                    #actionConversion[action[0]] = newId
                    counter += 1
                    self.dialogWindow.setValue(counter)
                
                self.dialogWindow.showMessage("Merging Properties")
                self.dialogWindow.setRange(0,len(propertyList))
                counter = 0
                for property in propertyList:
                    newId = adb.addNewProperty(property[1], property[2])
                    #propertyConversion[property[0]] = newId
                    counter += 1
                    self.dialogWindow.setValue(counter)
                
                linkList = []
                
                self.dialogWindow.showMessage("Merging Sentences")
                self.dialogWindow.setRange(0,len(sentences))
                counter = 0
                for sentence in sentences:
                                       
                    linkList.extend(sentence.getChildren())
                    newSentence = Sentence(sentence.getSentence())
                    #set new action ids
                    #pre
                    pre = sentence.getPreActionList()
                    for action in pre:
                        newAction = Action(action.getAction(),action.getKind(),order=action.getOrder())
                        newSentence.addPreAction(newAction)
                    #post
                    post = sentence.getPostActionList()
                    for action in post:
                        newAction = Action(action.getAction(),action.getKind(),order=action.getOrder())
                        newSentence.addPostAction(newAction)
                    
                    #set new property ids
                    for prop in sentence.getPropertys():
                        pro = sentence.getProperty(prop)
                        newProperty = Property(pro.getName(),pro.getValue())
                        newSentence.addProperty(newProperty)
                    
                    newSentence = adb.addNewSentence(newSentence,False)
                    sentenceConversion[sentence.getId()] = newSentence.getId()
                    
                    counter += 1
                    self.dialogWindow.setValue(counter)
                
                self.dialogWindow.showMessage("Merging Links")
                self.dialogWindow.setRange(0,len(linkList))
                counter = 0
                for link in linkList:
                    newLink = Link(sentenceConversion[link.getReplyToId()],sentenceConversion[link.getReplyId()],order=link.getOrder())
                    adb.addNewLink(newLink)
                    counter += 1
                    self.dialogWindow.setValue(counter)
                                
                self.dialogWindow.showMessage("Merging completed")
                adb.closeDatabase()
                sdb.closeDatabase()
                
                
                self.doneFunction()
                
            else:
                self.dialogWindow.showMessage("Failed to open: "+str(self.sourceDatabaseFilename))
        else:
            self.dialogWindow.showMessage("Failed to open: "+str(self.activeDatabaseFilename))

class MergeDatabases(ProgressDialog):
    
    def __init__(self,parent,activeDatabaseFilename,sourceDatabaseFilename):
        ProgressDialog.__init__(self,parent,"Merging databases","Source Database:",sourceDatabaseFilename,"Destination Database:",activeDatabaseFilename,"Merging..")
        self.activeDatabaseFilename = activeDatabaseFilename
        self.sourceDatabaseFilename = sourceDatabaseFilename
        
        thread = mergingThread(self.activeDatabaseFilename,self.sourceDatabaseFilename,self,self.mergeDone)
        
        thread.start()
    
        
    def mergeDone(self):       
        
        self.done(0)
        

        