from PyQt4 import QtCore, QtGui

from Database import Database

import shutil
import os.path
from datetime import datetime
import sys
import os.path
from Action import Action
from Link import Link
from Sentence import Sentence
from Property import Property
from threading import Thread
from ProgressDialog import ProgressDialog

class storyLinePrinterThread(Thread):
    
    def __init__(self,activeDatabaseFilename,destinationDatabaseFilename,dialogWindow,doneFunction): 
        Thread.__init__(self)
        self.activeDatabaseFilename = activeDatabaseFilename
        self.destinationDatabaseFilename = destinationDatabaseFilename
        
        
        self.dialogWindow = dialogWindow
        self.doneFunction = doneFunction
        
    def run(self):
        if not self.activeDatabaseFilename == None and len(self.activeDatabaseFilename) > 0 and os.path.isfile(self.activeDatabaseFilename):
            if not self.destinationDatabaseFilename == None and len(self.destinationDatabaseFilename) > 0 and not os.path.isfile(self.destinationDatabaseFilename):
                self.dialogWindow.setRange(0,0)
                self.dialogWindow.showMessage("Preparing print")
                adb = Database()
                dfile = open(self.destinationDatabaseFilename,"w")
                adb.openDatabase(self.activeDatabaseFilename)
                
                roots = adb.getRootSentences(False, False)
                lines = []
                for root in roots:
                    
                    
                    tree = self.makeTreeRecursive(root,adb,[])
                    
                    for item in tree:
                        item.insert(0,root.getId())
                        
                    for item in tree:
                        
                        for node in item:
                            line = ""
                            if node >= 0:
                                sentence = adb.getSentenceById(node, False, False)
                                avatar = ""
                                prop = sentence.getProperty('avatar_type')
                                if not prop == None:
                                    avatar = prop.getValue()
                                    
                                id = ""
                                if sentence.getId() < 10:
                                    id = " "*3+str(sentence.getId())                                
                                elif sentence.getId() < 100:
                                    id = " "*2+str(sentence.getId())  
                                
                                elif sentence.getId() < 1000:
                                    id = " "+str(sentence.getId())
                                
                                else:
                                    id = str(sentence.getId())   
                                    
                                line = id+": <"+str(avatar)+"> "+str(sentence.getSentence()+" \n")
                            else:
                                line = "<< Loop >> \n"
                            
                            lines.append(line)
                        lines.append("-"*40+"\n")
                    lines.append("*"*40+"\n")
                
                dfile.writelines(lines)
                                
                self.dialogWindow.showMessage("print completed")
                adb.closeDatabase()
                dfile.close()
                
                self.dialogWindow.setRange(0,1)
                self.dialogWindow.setValue(1)
                self.doneFunction()
                
            else:
                self.dialogWindow.showMessage("Failed to save: "+str(self.destinationDatabaseFilename))
        else:
            self.dialogWindow.showMessage("Failed to open: "+str(self.activeDatabaseFilename))
    
    def makeTreeRecursive(self,parent,db,doneList):
        children = parent.getChildrenOrdered()
        completeList = []
        for child in children:
            try:
                doneList.index(child.getReplyId())
                completeList.append([-2])
            except ValueError:
                newDoneList = doneList[:]
                newDoneList.append(child.getReplyId())
                childSentence = db.getSentenceById(child.getReplyId()) 
                returnList = self.makeTreeRecursive(childSentence,db,newDoneList)
                
                for item in returnList:
                    item.insert(0,child.getReplyId())
                    completeList.append(item)
        
        if len(children) == 0:
            completeList = [[]]
        
        return completeList
        
        
    
class StoryLinePrinter(ProgressDialog):
    
    def __init__(self,parent,activeDatabaseFilename,destinationDatabaseFilename):
        ProgressDialog.__init__(self,parent,"Exporting template","Destination Database:",destinationDatabaseFilename,"Source Database:",activeDatabaseFilename,"Exporting..")
        self.activeDatabaseFilename = activeDatabaseFilename
        self.destinationDatabaseFilename = destinationDatabaseFilename
        thread = storyLinePrinterThread(self.activeDatabaseFilename,self.destinationDatabaseFilename,self,self.printDone)
        
        thread.start()
       
    
            
        
    def printDone(self):       
        
        self.done(0)
